/*
 * adhoc_fileserver.cpp - Implementation of a personal ad-hoc fileserver
 * Copyright (C) 2005  Remko Troncon, Alban Crequy
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <qdir.h>
#include <qfileinfo.h>
#include <qmessagebox.h>

#include "common.h"
#include "psiaccount.h"
#include "psicon.h"
#include "adhoc_fileserver.h"
#include "filesharingdlg.h"
#include "xmpp_xdata.h"

using namespace XMPP;

bool AHFileServer::isAllowed(const Jid& j) const
{
	/*
	 * The ACL on a per file basis is not done here.
	 *
	 * FIXME: return true only if the jid is in the contact list
	 *  */
	return true;
	//return manager()->account()->jid().compare(j,false);
}

bool AHFileServer::ACL_check(QStringList ACL, const Jid& j) const
{
	/* ACL check for a specific file
	 * */
	bool granted = false;
	for ( QStringList::Iterator it = ACL.begin(); it != ACL.end(); ++it ) {
		*it;
		if (j.compare(*it, false))
			granted = true;
	}
	return granted;
}

AHCommand AHFileServer::execute(const AHCommand& c, const Jid& requester)
{
	// Extract the file
	QString file;
	if (c.hasData()) {
		QString fileName, curDir;
		QStringList ACL;
		XData::FieldList fl = c.data().fields();
		for (unsigned int i=0; i < fl.count(); i++) {
			if (fl[i].var() == "file" && !(fl[i].value().isEmpty())) {
				file = fl[i].value().first();
			}
		}

		QListViewItemIterator it = this->m->account()->psi()->fsdlg()->getSharedItemsIterator();
		fileName = QString("");
		ACL = QStringList();
		while ( it.current() ) {
			if (file == it.current()->text(0)) {
				fileName = it.current()->text(1);
				ACL = QStringList::split(QRegExp(QString(",")), it.current()->text(2), FALSE);
			}
			++it;
		}
		
		if (fileName != QString("")) {
			/* Warning: the following lines will send the file
			 * Check the ACL first! */
			if (ACL_check(ACL, requester)) {
				QStringList l(fileName);
				manager()->account()->sendFiles(requester,l,true);
			}
			else {
				/* FIXME */
				QMessageBox::information(0,
						QObject::tr("Error"),
						QObject::tr("Remote contact does not have the permission"));
			}
		}
		else {
			/* FIXME */
			QMessageBox::information(0, QObject::tr("Error"), QObject::tr("Invalid query from remote contact"));
		}
		return AHCommand::completedReply(c);
	}
	else {
		// Return a form with a filelist
		XData form;
		form.setTitle(QObject::tr("Choose file"));
		form.setInstructions(QObject::tr("Choose a file"));
		form.setType(XData::Data_Form);
		XData::FieldList fields;

		XData::Field files_field;
		files_field.setType(XData::Field::Field_ListSingle);
		files_field.setVar("file");
		files_field.setLabel(QObject::tr("File"));
		files_field.setRequired(true);

		XData::Field::OptionList file_options;

		QListViewItemIterator it = this->m->account()->psi()->fsdlg()->getSharedItemsIterator();
		while ( it.current() ) {
			QStringList ACL = QStringList::split(QRegExp(QString(",")), it.current()->text(2), FALSE);
			/* The requester can only see a subset of the shared files:
			 * We don't want the requester see a file it can't download */
			if (ACL_check(ACL, requester)) {
				XData::Field::Option file_option;
				file_option.label = it.current()->text(0);
				file_option.value = it.current()->text(0);
				file_options += file_option;
			}
			++it;
		}
		files_field.setOptions(file_options);
		fields += files_field;
		
		form.setFields(fields);

		return AHCommand::formReply(c, form);
	}
}

