/*
 * filesharingdlg.cpp - dialogs for manipulating list of shared files
 * Copyright (C) 2005  Alban Crequy
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"ui_filesharing.h"
#include"ui_filesharingmodify.h"

#include"filesharingdlg.h"

#include<qpushbutton.h>
#include<qlistview.h>
#include<qlineedit.h>
#include<qcheckbox.h>
#include<qlabel.h>
#include<qlayout.h>
#include<qgroupbox.h>
#include<qcombobox.h>
#include<qradiobutton.h>
#include<qbuttongroup.h>
#include<qtabwidget.h>
#include<qfiledialog.h>
#include<qwhatsthis.h>
#include<qurl.h>
#include<qca.h>
#include"psicon.h"
#include"psiaccount.h"
#include"common.h"
#include"im.h"
#include"xmpp_tasks.h"
#include"proxy.h"
#include"sslcertdlg.h"
#include"busywidget.h"
#include"iconwidget.h"
#include"fancylabel.h"
#include"base64.h"

using namespace XMPP;

//----------------------------------------------------------------------------
// FileSharingItem
//----------------------------------------------------------------------------
FileSharingItem::FileSharingItem(QListView *par, QString fileLabel,
		QString filePath, QString fileACL)
:QCheckListItem(par,"",CheckBox)
{
	setFileLabel(fileLabel);
	setFilePath(filePath);
	setFileACL(fileACL);
	updateInfo();
}

void FileSharingItem::updateInfo()
{
	setText(0, p_fileLabel);
	setText(1, p_filePath);
	setText(2, p_fileACL);
}

int FileSharingItem::rtti() const
{
	return 8151;
}

QString FileSharingItem::getFileLabel()
{
	return QString(this->p_fileLabel);
}

void FileSharingItem::setFileLabel(QString _fileLabel)
{
	this->p_fileLabel = QString(_fileLabel);
}


QString FileSharingItem::getFilePath()
{
	return QString(this->p_filePath);
}

void FileSharingItem::setFilePath(QString _filePath)
{
	this->p_filePath = QString(_filePath);
}


QString FileSharingItem::getFileACL()
{
	return QString(this->p_fileACL);
}

QStringList FileSharingItem::getFileACL_list()
{
	QStringList ACL_list = QStringList::split(QRegExp(QString(",")), this->p_fileACL, FALSE);
	return ACL_list;
}

void FileSharingItem::setFileACL(QString _fileACL)
{
	this->p_fileACL = QString(_fileACL);
}

void FileSharingItem::addFileACL_Jid(QString jid)
{
	this->p_fileACL = QString(jid);
}

void FileSharingItem::delFileACL_Jid(QString jid)
{
	this->p_fileACL = QString("");
}


//----------------------------------------------------------------------------
// FileSharingDlg
//----------------------------------------------------------------------------
FileSharingDlg::FileSharingDlg(PsiCon *_psi, const char *name)
:FileSharingUI(0, name, false, psi_dialog_flags)
{
	psi = _psi;

	setCaption(CAP(caption()));

	// setup signals
	connect(pb_add, SIGNAL(clicked()), SLOT(add()));
	connect(pb_modify, SIGNAL(clicked()), SLOT(modify()));
	connect(pb_remove, SIGNAL(clicked()), SLOT(remove()));
	connect(pb_close, SIGNAL(clicked()), SLOT(close()));

	connect(lv_sharedfiles, SIGNAL(doubleClicked(QListViewItem *)), SLOT(modify(QListViewItem *)));
	connect(lv_sharedfiles, SIGNAL(selectionChanged(QListViewItem *)), SLOT(qlv_selectionChanged(QListViewItem *)));

	lv_sharedfiles->setAllColumnsShowFocus(true);
	lv_sharedfiles->setResizeMode(QListView::LastColumn);

	if(lv_sharedfiles->childCount() > 0)
		lv_sharedfiles->setSelected(lv_sharedfiles->firstChild(), true);
	else
		qlv_selectionChanged(0);
}

FileSharingDlg::~FileSharingDlg()
{
}

QListViewItemIterator FileSharingDlg::getSharedItemsIterator()
{
	return QListViewItemIterator(lv_sharedfiles);
}

void FileSharingDlg::qlv_selectionChanged(QListViewItem *lvi)
{
	FileSharingItem *i = (FileSharingItem *)lvi;
	bool ok = i ? true: false;

	pb_modify->setEnabled(ok);
	pb_remove->setEnabled(ok);
}

void FileSharingDlg::add()
{
	QString fileName;
	QString filePath = QFileDialog::getOpenFileName(option.lastPath, tr("All files (*)"), this, 0, tr("Choose a file"));
	
	
	if (filePath == NULL) /* The user press Cancel */
		return;
	if(filePath.isEmpty())
		return;

	QFileInfo fi(filePath);

	if(!fi.exists()) {
		QMessageBox::information(this, tr("Error"), tr("The file specified does not exist."));
		return;
	}
	option.lastPath = QString(fi.dirPath());
	fileName = QString(fi.fileName());

	QListViewItem newItem = new FileSharingItem(lv_sharedfiles,
			fileName, filePath, QString(""));

	lv_sharedfiles->setCurrentItem(&newItem);
}

void FileSharingDlg::modify()
{
	modify(lv_sharedfiles->currentItem());
}

void FileSharingDlg::modify(QListViewItem *lvi)
{
	FileSharingItem *i = (FileSharingItem*) lvi;
	FileSharingModifyDlg *w = new FileSharingModifyDlg(psi, NULL, lv_sharedfiles, i, 0);
	w->show();
}

void FileSharingDlg::remove()
{
	FileSharingItem *i = (FileSharingItem *)lv_sharedfiles->currentItem();
	if(!i)
		return;

	/* FIXME : confirmation dialog **
	AccountRemoveDlg *w = new AccountRemoveDlg(psi->proxy(), i->pa->userAccount());
	int n = w->exec();
	if(n != QDialog::Accepted) {
		delete w;
		return;
	}
	delete w;
	*/
	qlv_selectionChanged(NULL);
	delete i;
}

//----------------------------------------------------------------------------
// FileSharingModifyDlg
//----------------------------------------------------------------------------

FileSharingModifyDlg::FileSharingModifyDlg(PsiCon *_psi, QWidget *parent, QListView *_lv_sharedfiles,
		FileSharingItem *_fileItem, const char *name)
:FileSharingModifyUI(parent, name, false, WDestructiveClose)
{
	psi = _psi;
	psi->dialogRegister(this);
	lv_sharedfiles = _lv_sharedfiles;
	fileItem = (FileSharingItem*)lv_sharedfiles->currentItem();

	setCaption(CAP(caption()));

	connect(pb_cancel, SIGNAL(clicked()), SLOT(close()));
	connect(pb_save, SIGNAL(clicked()), SLOT(save()));
	connect(pb_open, SIGNAL(clicked()), SLOT(on_pb_open_clicked()));
	connect(pb_add, SIGNAL(clicked()), SLOT(on_pb_add_clicked()));
	connect(pb_remove, SIGNAL(clicked()), SLOT(on_pb_remove_clicked()));
	connect(file_label, SIGNAL(textChanged(const QString &)), SLOT(setModifyButton(const QString &)));

	file_label->setText(fileItem->getFileLabel());
	file_path->setText(fileItem->getFilePath());
	p_ACL = QString(fileItem->getFileACL());

	QStringList ACL_list = fileItem->getFileACL_list();
	
	for (QStringList::Iterator it = ACL_list.begin(); it != ACL_list.end(); ++it ) {
		new QListViewItem(lv_grantedjid, QString(*it));
	}
	
	file_label->setFocus();
}

FileSharingModifyDlg::~FileSharingModifyDlg()
{
	psi->dialogUnregister(this);
}

void FileSharingModifyDlg::save()
{
	p_ACL = QString("");
			
	QPtrList<QListViewItem> lst;
	QListViewItemIterator it(lv_grantedjid);
	while ( it.current() ) {
		if (p_ACL.isEmpty())
			p_ACL = it.current()->text(0);
		else
			p_ACL += QString(",") + it.current()->text(0);
		++it;
	}

	fileItem->setFileLabel(file_label->text());
	fileItem->setFilePath(file_path->text());
	fileItem->setFileACL(p_ACL);
	fileItem->updateInfo();

	close();
}

void FileSharingModifyDlg::setModifyButton(const QString &s)
{
	pb_save->setEnabled(!s.isEmpty());
}

void FileSharingModifyDlg::on_pb_open_clicked()
{
	QString newFileName;
	QString newFilePath = QFileDialog::getOpenFileName(option.lastPath, tr("All files (*)"), this, 0, tr("Choose a file"));
	
	
	if (newFilePath == NULL) /* The user press Cancel */
		return;
	if(newFilePath.isEmpty())
		return;

	QFileInfo fi(newFilePath);

	if(!fi.exists()) {
		QMessageBox::information(this, tr("Error"), tr("The file specified does not exist."));
		return;
	}
	option.lastPath = QString(fi.dirPath());

	file_label->setText(QString(fi.fileName()));
	file_path->setText(QString(newFilePath));
}

void FileSharingModifyDlg::on_pb_add_clicked()
{
	QString str = jid_to_add->text();
	if (str == NULL)
		return;
	if (str.isEmpty())
		return;

	new QListViewItem(lv_grantedjid, QString(str));
}

void FileSharingModifyDlg::on_pb_remove_clicked()
{
	QListViewItem *i = lv_grantedjid->currentItem();
	if(!i)
		return;
	delete i;
}

