/*
 * filesharing.h - dialogs for manipulating list of shared files
 * Copyright (C) 2005  Alban Crequy
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef FILESHARINGDLG_H
#define FILESHARINGDLG_H

#include<qca.h>
#include<qlistview.h>
#include"ui_filesharing.h"
#include"ui_filesharingmodify.h"
#include"ui_pgpkey.h"
#include"xmpp.h"
#include"openpgp.h"

namespace XMPP
{
	class Jid;
	class Client;
}

using namespace XMPP;

class PsiCon;
class PsiAccount;
class QListViewItem;
class UserAccount;
class ProxyManager;
class ProxyChooser;

class FileSharingItem : public QCheckListItem
{
public:
	FileSharingItem(QListView *par, QString fileLabel,
			QString filePath, QString fileACL);

	void updateInfo();

	int rtti() const;

	QString getFileLabel();
	void setFileLabel(QString _fileLabel);
	
	QString getFilePath();
	void setFilePath(QString _filePath);
	
	QString getFileACL(); /* Get ACL as a string for the user */
	QStringList getFileACL_list();
	void setFileACL(QString _fileACL);

	void addFileACL_Jid(QString jid);
	void delFileACL_Jid(QString jid);

private:
	QString p_fileLabel;
	QString p_filePath;
	QString p_fileACL;
};

class FileSharingDlg : public FileSharingUI
{
	Q_OBJECT
public:
	FileSharingDlg(PsiCon *, const char *name=0);
	~FileSharingDlg();
	QListViewItemIterator getSharedItemsIterator();

private slots:
	void qlv_selectionChanged(QListViewItem *);
	void add();
	void modify();
	void modify(QListViewItem *);
	void remove();

private:
	PsiCon *psi;
};

class FileSharingModifyDlg : public FileSharingModifyUI
{
	Q_OBJECT
public:
	FileSharingModifyDlg(PsiCon *, QWidget *parent=0, QListView *_lv_sharedfiles=0,
			FileSharingItem *_fileItem=0, const char *name=0);
	~FileSharingModifyDlg();

private slots:
	void save();
	void setModifyButton(const QString &);
	void on_pb_open_clicked();
	void on_pb_add_clicked();
	void on_pb_remove_clicked();

private:
	PsiCon *psi;
	QListView *lv_sharedfiles;
	FileSharingItem *fileItem;
	QString p_ACL;
};

#endif
